@@ -61,8 +61,7 @@
 
 (define-method (initialize (ws <canvas-wrapset>) initargs)
   (next-method ws (cons #:module (cons '(gnome gw canvas) initargs)))
+  ;; ?
   (if #f
       (wrap-custom-pointer! "GnomeCanvasPoints"))
-  (add-type-alias! ws "double" 'double)
-  (add-type-rule! ws '(("double*" "*")) '(double out))
   (load-defs-with-overrides ws "gnome/defs/libgnomecanvas.defs"))
